// ExperSwarm.m

#import "ExperSwarm.h"

	
// implementation of ExperSwarm object

@implementation ExperSwarm

+ createBegin: aZone
  {
  ExperSwarm *obj;
  id <ProbeMap> experProbeMap;

  obj = [super createBegin: aZone];

// probe map for ExperSwarm
  experProbeMap = [EmptyProbeMap createBegin: aZone];
  [experProbeMap setProbedClass: [self class]];
  experProbeMap = [experProbeMap createEnd];

  [experProbeMap addProbe: [probeLibrary getProbeForVariable: "minStartBudget"
                                   inClass: [self class]]];
  [experProbeMap addProbe: [probeLibrary getProbeForVariable: "maxStartBudget"
                                   inClass: [self class]]];
  [experProbeMap addProbe: [probeLibrary getProbeForVariable: "incStartBudget"
                                   inClass: [self class]]];

  [probeLibrary setProbeMap: experProbeMap For: [self class]];

  return obj;
  }

- createEnd
  {
  return [super createEnd];
  }

- buildObjects
  {
  [super buildObjects];

  CREATE_ARCHIVED_PROBE_DISPLAY (self);

// Allow the user to alter experiment parameters
  [controlPanel setStateStopped];

// build the Graph for model results
  spendingGraph = [Graph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME(spendingGraph);
  spendingGraph = [spendingGraph createEnd];
  [spendingGraph setTitle: "Average Spending"];
  [spendingGraph setWidth:500 Height:400];
  [spendingGraph setAxisLabelsX:"Start Budget" Y:"Average Spending"];

  spending=[spendingGraph createElement];
  [spending setLabel: "average spending on the market"];
  [spending setColor: "blue"];

  return self;
  }

- run
  {
  id <LispArchiver> outFile;
  SimulationData * simLoop;
  id <LispArchiver> archiver;
  id <Index> i=nil;
  int sum=0;
  Integer * spendingElement;

// start running simulation: for each possible starting budget
  for (setStartBudget=minStartBudget;
  		setStartBudget<=maxStartBudget;setStartBudget+=incStartBudget)
  	{
printf("this round, StartBudget is %d \n",setStartBudget);
		
//create setup file for Model Swarm

	simLoop=[SimulationData create:[self getZone]];
	[simLoop initPara:setStartBudget];
	outFile=[LispArchiver create:self setPath:"loop.scm"];
	[outFile putShallow:"modelSwarm" object:simLoop];
	[outFile sync];
	[outFile drop];

// create listOfAllSpending
	listOfAllSpending=[List create:[self getZone]];

//load the data for the modelSwarm
	archiver=[LispArchiver create:globalZone setPath:"parameters.scm"];
	if((modelSwarm=[archiver getWithZone:globalZone key:"modelSwarm"])==nil)
		{
		raiseEvent(InvalidOperation,"can't find file or key\n");
		}
	[archiver drop];

//load the data modified for the experiment (must come after the
//data for the model as it modifies the values of some of them)
//archiver loads the data from the file loop.scm. They are a SimulationData
//object, so use the setPara:model method to initialise the modelSwarm
	archiver=[LispArchiver create:globalZone setPath:"loop.scm"];
	if((simLoop=[archiver getWithZone:globalZone key:"modelSwarm"])==nil)
		{
		raiseEvent(InvalidOperation,"can't find file or key\n");
		}
	[simLoop setPara:modelSwarm];
	[archiver drop];
	[simLoop drop];

	[modelSwarm buildObjects];
	[modelSwarm buildActions];

// run modelSwarm until it finishes
		while ([modelSwarm checkToStop]==0)
		  {
	      [modelSwarm marketDay];
  		  [modelSwarm increaseTime];
  		  [modelSwarm checkToStop];
    	  }

// put values in listOfAllSpending from modelSwarm
	listOfAllSpending=[modelSwarm getAllSpending];
// get average spending to use in graph
	i=[listOfAllSpending begin:[self getZone]];
	while ((spendingElement=[i next])!=nil)
		{
		sum+=[spendingElement getMyValue];
		}
	averageSpending=sum/[listOfAllSpending getCount];
printf("averageSpending %f\n",averageSpending);

// add data for graph
	[spending addX:setStartBudget Y:averageSpending];
    }  // end of for loop on startBudget

// end of experiment, draw graph and stop control panel
  printf("End of the experiment\n");
  [spendingGraph pack];	
  [controlPanel setStateStopped];

  return self;
  }  

- activateIn: swarmContext
  {
  [super activateIn: swarmContext];

  return [self getActivity];
  }


@end

