// ObserverSwarm.m

#import "ObserverSwarm.h"

@implementation ObserverSwarm

+createBegin: (id) aZone
	{
	ObserverSwarm * obj;
	id <ProbeMap> probeMap;

	obj = [super createBegin: aZone];

// probe map for observer
	probeMap=[EmptyProbeMap createBegin:aZone];
	[probeMap setProbedClass:[self class]];
	probeMap=[probeMap createEnd];

	[probeMap addProbe:[probeLibrary getProbeForVariable:"displayFrequency"
									inClass:[self class]]];
	[probeMap addProbe:[probeLibrary getProbeForVariable:"displayConsumerName"
									inClass:[self class]]];

	[probeLibrary setProbeMap: probeMap For:[self class]];									

	return obj;
	}

-createEnd
	{
	return [super createEnd];
	}

-buildObjects
	{
	id <LispArchiver> archiver;
  
	[super buildObjects];

// create the modelSwarm, reading parameters from a file
	archiver=[LispArchiver create:[self getZone] setPath:"parameters.scm"];
	if((modelSwarm=[archiver getWithZone:[self getZone] key:"modelSwarm"])==nil)
		{
		raiseEvent(InvalidOperation,"can't find file or key\n");
		}
	[archiver drop];

// create probes for modelSwarm and observerSwarm
	CREATE_ARCHIVED_PROBE_DISPLAY (modelSwarm);
	CREATE_ARCHIVED_PROBE_DISPLAY (self);

// set control panel to state stopped
	[controlPanel setStateStopped];

// Then we ask the model to build itself.
	[modelSwarm buildObjects];

// create graphics
// the spendingGraph shows the averag, total, min, max spending of
// consumers during the game
	spendingGraph = [EZGraph createBegin: self];
	SET_WINDOW_GEOMETRY_RECORD_NAME(spendingGraph);
	[spendingGraph setTitle: "Agents' spending"];
	[spendingGraph setAxisLabelsX: "Time" Y: "Spending"];
	spendingGraph = [spendingGraph createEnd];						 
	[spendingGraph createAverageSequence: "Average Spending"
		withFeedFrom: [modelSwarm getListOfConsumers] andSelector:M(getSpending)];
	[spendingGraph createTotalSequence: "Total Spending"
		withFeedFrom: [modelSwarm getListOfConsumers] andSelector:M(getSpending)];
	[spendingGraph createMinSequence: "Minimum Spending"
		withFeedFrom: [modelSwarm getListOfConsumers] andSelector:M(getSpending)]; 

// the consumerGraph shows the spending and the number of times
// consumer 1 went to the market
	consumerGraph=[EZGraph createBegin:self];
	SET_WINDOW_GEOMETRY_RECORD_NAME(consumerGraph);
	[consumerGraph setTitle:"A consumer"];
	[consumerGraph setAxisLabelsX: "Time" Y: "Visits/Spending"];
	consumerGraph=[consumerGraph createEnd];
	[consumerGraph createSequence: "Went to the market"
		withFeedFrom:[modelSwarm getConsumer:displayConsumerName] andSelector:M(getVisit)];
	[consumerGraph createSequence:"Spent"
		withFeedFrom:[modelSwarm getConsumer:displayConsumerName] andSelector:M(getSpending)];
	return self;
	}

-buildActions
	{
	[super buildActions];
	[modelSwarm buildActions];

// define displayActions
	displayActions = [ActionGroup create: self];
// update the graphs
	[displayActions createActionTo: spendingGraph message:M(step)];
	[displayActions createActionTo: consumerGraph message:M(step)];
	[displayActions createActionTo: self message:M(observerCheckToStop)];
	[displayActions createActionTo: actionCache message: M(doTkEvents)];
// define displaySchedule
	displaySchedule = [Schedule createBegin: self];
	[displaySchedule setRepeatInterval: 2]; 
	displaySchedule = [displaySchedule createEnd];
	[displaySchedule at: 1 createAction: displayActions];
	return self;
	}

-activateIn: (id) swarmContext
	{
	[super activateIn: swarmContext];
	[displaySchedule activateIn: self];
	[modelSwarm activateIn: self];
	return [self getSwarmActivity];
	}

-observerCheckToStop
	{
	if ([modelSwarm checkToStop]==1)
		{
		[controlPanel setStateStopped];
		}
	return self;
	}
@end

