// ModelSwarm.m

#import "ModelSwarm.h"

@implementation ModelSwarm

+createBegin: (id) aZone
	{
	ModelSwarm * obj;
	id <ProbeMap> modelProbeMap;
// call the createBegin method of the superClass
	obj = [super createBegin: aZone];
// create the probemap for the model
	modelProbeMap=[EmptyProbeMap createBegin:aZone];
	[modelProbeMap setProbedClass:[self class]];
	modelProbeMap=[modelProbeMap createEnd];

	[modelProbeMap addProbe:[probeLibrary getProbeForVariable:"maxTime"
				inClass:[self class]]];
	[modelProbeMap addProbe:[probeLibrary getProbeForVariable:"numberOfConsumers"
				inClass:[self class]]];
	[modelProbeMap addProbe:[probeLibrary getProbeForVariable:"startBudget"
				inClass:[self class]]];
	[modelProbeMap addProbe:[probeLibrary getProbeForVariable:"maxBudget"
				inClass:[self class]]];
	[modelProbeMap addProbe:[probeLibrary getProbeForVariable:"worldWidth"
				inClass:[self class]]];
	[modelProbeMap addProbe:[probeLibrary getProbeForVariable:"worldHeight"
				inClass:[self class]]];
	[probeLibrary setProbeMap:modelProbeMap For:[self class]];
	return obj;
	}

-createEnd
  {
  return [super createEnd];
  }

-buildObjects
	{
	int i,x,y;
	int name;
	id <ProbeMap> consumerProbe;

	[super buildObjects];
	
// set the position of the market in the center of the world
	xMin=(worldWidth-sizeOfMarket)/2;
	xMax=xMin+sizeOfMarket;
	yMin=(worldHeight-sizeOfMarket)/2;
	yMax=yMin+sizeOfMarket;

// initialise the world as a grid2d and fill it with "nil" objects
	world=[Grid2d create:[self getZone] setSizeX:worldWidth Y:worldHeight];
	[world fillWithObject:nil];
// initialise market as a Discrete2d and fill it with a value object
// to get the yellow square representing the market
	market=[Discrete2d create:[self getZone] setSizeX:worldWidth Y:worldHeight];
	for (y=yMin; y<yMax; y++)
		{
    	for (x=xMin; x<xMax; x++)
			{
			[market putValue:2 atX: x Y: y];
			}
		}

// create the list of consumers
	listOfConsumers=[List create:[self getZone]];

// iterate over all possible consumers (from 1 to numberOfConsumers)
	for (i=1;i<=numberOfConsumers;++i)
		{
		Consumer * aConsumer;
// name of consumer=index i
		name=i;
// create the consumers
	    aConsumer=[Consumer create: [self getZone]];
   		[aConsumer setConsumerName:name MaxBudget:maxBudget StartBudget:startBudget Goer:0];
// set their positions to -999
		[aConsumer setPositionX:-999 Y:-999];
// create the map and arrays (the array is created with a size of
// maxTime+1, as time starts at 0
		[aConsumer createMapOfSpending:[self getZone]];
		[aConsumer createArrayOfVisits:[self getZone]:maxTime+1];
// position the consumers in the world, excluding market
		[self findPositionInWorld:world For:aConsumer ExcludeMarket:1];
// probe for consumer
		consumerProbe=[EmptyProbeMap createBegin:[self getZone]];
		[consumerProbe setProbedClass:[Consumer class]];
		consumerProbe=[consumerProbe createEnd];
		[consumerProbe addProbe:[probeLibrary getProbeForVariable:"myName"
				inClass:[Consumer class]]];
		[probeLibrary setProbeMap:consumerProbe For:[Consumer class]];
// add consumer to the list
		[listOfConsumers addFirst:aConsumer];
		}
	return self;
	}

// this method positions the consumers in the world, excluding the market,
// or only in the market according to the value of the last argument
-findPositionInWorld:(id <Grid2d>)aWorld For:(Consumer *)aGuy ExcludeMarket:(int)exclude
	{
	int trialX,trialY;
// set trialX and trialY to negative values (not in the world) to start the
// while loop
	trialX=-999;
	trialY=-999;
// if the consumers have already been put on a space (positionX and positionY are not
// -999, then put a nil object at their current position
	if (([aGuy getPositionX]>=0)&&([aGuy getPositionY]>=0))
		{
		[aWorld putObject:nil atX:[aGuy getPositionX] Y:[aGuy getPositionY]];
		}
// put consumers in the world randomly. if exclude=1, then the part of
// the world corresponding to the market is excluded from the
// possibilities, if exclude=0, the consumer is put in the market
// when choosing a value for the position of the consumer, we need
// to check that: (1) both coordinates are positive, (2) they are
// in the part of the world where they should be, (3) there is not
// already a player in the spot. This explain why the while condition
// is quite complex
	if (exclude==1)
		{
		while( ((trialX<0)&&(trialY<0)) ||
			((trialX>=xMin)&&(trialX<=xMax)&&(trialY>=yMin)&&(trialY<=yMax)) ||
			([world getObjectAtX:trialX Y:trialY]!=nil) )
			{
			trialX=[uniformIntRand getIntegerWithMin:0 withMax:worldWidth-1];
			trialY=[uniformIntRand getIntegerWithMin:0 withMax:worldHeight-1];
			}
		}
	else if (exclude==0)
		{
		while ( ((trialX<0)&&(trialY<0)) ||
			([world getObjectAtX:trialX Y:trialY]!=nil) )
			{
			trialX=[uniformIntRand getIntegerWithMin:xMin withMax:xMax];
			trialY=[uniformIntRand getIntegerWithMin:yMin withMax:yMax];
			}
		}
	else
		{
		printf("wrong value for exclude!\n");
		exit(0);
		}
	[aGuy setPositionX:trialX Y:trialY];
	[aWorld putObject:aGuy atX:trialX Y:trialY];
	return self;
	}


-marketDay
	{
	int go;
	int spending;
	int budget;
	id <Index> i=nil; // to iterate over the list of consumers
	Consumer * listElement;

// iterate over the list of consumers
// first, create the index i
	i=[listOfConsumers begin:[self getZone]];
	while ((listElement=[i next])!=nil)
		{
// update the budget of the consumer
		budget=[listElement findBudget];

// is he going to the market?
		go=[listElement goToTheMarket];
	    if (go)
       		{
	        spending=[listElement spend];
// add 1 to arrayOfVisits, at the position corresponding to
// current modelTime
			[listElement updateVisits:modelTime:1];
// add spending at the key modelTime in the mapOfSpending
			[listElement updateSpending:modelTime:spending];
// now, print a report of the consumer's actions
			printf("This is time %d\n",modelTime);
	        printf("I am consumer %d\n",[listElement getConsumerName]);
			printf("My current budget is %d\n",[listElement getBudget]);
			printf("Did I go to the market? %d (from array)\n",[listElement getVisitValue:modelTime]);
			printf("I spent %d (from map)\n",[listElement getSpendingValue:modelTime]);
       		printf("I have %d of currency left.\n",[listElement calculateRemainingBudget]);
// put in position in world on market space
			[self findPositionInWorld:world For:listElement ExcludeMarket:0];
       		}
	    else
       		{
// add 0 to arrayOfVisits at modelTime
			[listElement updateVisits:modelTime:0];
// add 0 to mapOfSpending at modelTime
			[listElement updateSpending:modelTime:0];
// print consumer's state
			printf("This is time %d\n",modelTime);
	        printf("I am consumer %d\n",[listElement getConsumerName]);
			printf("My current budget is %d\n",[listElement getBudget]);
			printf("Did I go to the market? %d (from array)\n",[listElement getVisitValue:modelTime]);
	        printf("I have %d of currency left.\n",[listElement getBudget]);
// put in position in world outside market
			[self findPositionInWorld:world For:listElement ExcludeMarket:1];
       		}
		}// end of iteration on listOfConsumers
// it is good practice to drop unused objects like indexes when they
// are no longer needed
		[i drop];
	return self;
	}

-increaseTime
	{
// at the end of a period, modelTime need to be increased by 1
	++modelTime;
	return self;
	}
-(int)checkToStop
	{
// if modelTime<maxTime, then notFinished is 1, otherwhise it
// is 0 (return respectively 0 or 1 for the observer)
	if (modelTime<=maxTime)
		{
		notFinished=1;
		return 0;
		}
	else
		{
		notFinished=0;
		return 1;
		}
	}

	
-buildActions
	{
// create an action group for the actions the model need to
// perform at each time period
// first, we tell people to go to the market (marketDay)
// then we increase the time of the model (increaseTime)
// finally we check whether this time is still valid (checkToStop)
	modelActions=[ActionGroup createBegin:self];
	modelActions=[modelActions createEnd];
	[modelActions createActionTo:self message:M(marketDay)];
	[modelActions createActionTo:self message:M(increaseTime)];
	[modelActions createActionTo:self message:M(checkToStop)];

// now schedule the actions in time
	modelSchedule = [Schedule createBegin: self];
	[modelSchedule setRepeatInterval:2];
	modelSchedule = [modelSchedule createEnd];
	[modelSchedule at: 0 createAction: modelActions];
	return self;
	}

-activateIn: (id) swarmContext
	{
	[super activateIn: swarmContext];
	[modelSchedule activateIn: self];
	
	return [self getActivity];
	}

// methods to pass parameters to other classes
-getListOfConsumers
	{
	return listOfConsumers;
	}

-getConsumer:(int)name
	{
// note that elements are entered in the list using the addFirst method,
// so the first element is the latest agent created.
	return [listOfConsumers atOffset:[listOfConsumers getCount]-name];
	}

-(int)getCurrentTime
	{
	return modelTime;
	}
-(int)getWorldWidth
	{
	return worldWidth;
	}
-(int)getWorldHeight
	{
	return worldHeight;
	}
-getWorld
	{
	return world;
	}
-getMarket
	{
	return market;
	}
@end